/// t2c_rcat.h
/// Requirement catalogue support - declarations.

/////////////////////////////////////////////////////////////////////////////
// Copyright 2009-2010 
// Institute for System Programming of the Russian Academy of Sciences (ISPRAS)
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
//    http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License. 
/////////////////////////////////////////////////////////////////////////////

#ifndef T2C_RCAT_H_1048_INCLUDED
#define T2C_RCAT_H_1048_INCLUDED

#include <stddef.h>

#ifdef __cplusplus
extern "C" {
#endif

///////////////////////////////////////////////////////////////////////////
/// CReqCat is used to access the catalogue of requirements.
struct CStringMap_; 
typedef struct CStringMap_ CReqCat;

/////////////////////////////////////////////////////////////////////////////
//  Defines 
/////////////////////////////////////////////////////////////////////////////

/// Default extension of files containing catalogues of requirements
#define DEF_RCAT_EXT ".xml"

/////////////////////////////////////////////////////////////////////////////
//  Methods 
/////////////////////////////////////////////////////////////////////////////

/// Create a new CReqCat structure, requirements from the specified files 
/// ("rcat-files") and return a pointer to the structure.
/// Names of the files are listed in NULL-terminated array 'rcat_names' (empty
/// strings are ignored). DEF_RCAT_EXT (see above) is appended to each file name
/// before trying to open the file.
/// The files are looked for in 'rcat_dir' directory.
/// 
/// The function returns NULL in case of failure. Additionally, it may provide
/// some more information about the failure in '*error_descr' if 'error_descr'
/// is not NULL. If it does, '*error_descr' should be freed by the caller when
/// no longer needed.
/// 
/// If none of the files exist, it is not considered an error. A catalogue with
/// no requirements will be created in this case.
/// 
/// It is OK for a requirement to be present in two or more rcat-files. All these
/// occurences of it will be loaded. 
CReqCat* 
t2c_rcat_create(const char* rcat_names[], const char* rcat_dir, char** error_descr);

/// Return number of requirements stored in the catalogue.
/// Note that if a requirement was present in two or more loaded rcat-files, it 
/// will be counted that number of times.
size_t
t2c_rcat_get_size(const CReqCat* rc);

/// Lookup the requirement with id 'id_what' in the catalogue, return its text
/// if found, NULL otherwise.
/// The returned string is owned by the catalogue and will be freed automatically
/// when the catalogue (CReqCat structure) is destroyed.
/// If there is more than one record with the specified id in the catalogue,
/// it is not defined which of these will be fetched by the function.
const char*
t2c_rcat_lookup(CReqCat* rc, const char* id_what);

/// Destroy the CReqCat structure.
void
t2c_rcat_destroy(CReqCat* rc);

#ifdef __cplusplus
}
#endif

#endif // T2C_RCAT_H_1048_INCLUDED
