/// t2c_api.c
/// Implementation of T2C API specific to the testing framework used (like 
/// TET, etc.)

/////////////////////////////////////////////////////////////////////////////
// Copyright 2009-2010 
// Institute for System Programming of the Russian Academy of Sciences (ISPRAS)
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
//    http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License. 
/////////////////////////////////////////////////////////////////////////////

#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <errno.h>
#include <assert.h>

#include <t2c_api.h>
#include <tet_api.h>

/////////////////////////////////////////////////////////////////////////////
// Implementation: public functions
/////////////////////////////////////////////////////////////////////////////
int
tet_code_for_t2c_code(ET2CTestResult tres)
{
    switch (tres)
    {
    case T2C_TR_ERROR:
        assert(0 && "T2C_TR_ERROR passed to tet_code_for_t2c_code()");
        return TET_UNRESOLVED; // release mode only
        break;
        
    case T2C_TR_PASSED:
        return TET_PASS;
        break;
    
    case T2C_TR_FAILED:
        return TET_FAIL;
        break;
    
    case T2C_TR_UNRESOLVED:
        return TET_UNRESOLVED;
        break;
    
    case T2C_TR_UNSUPPORTED:
        return TET_UNSUPPORTED;
        break;
    
    case T2C_TR_UNTESTED:
        return TET_UNTESTED;
        break;
    
    case T2C_TR_UNINITIATED:
        return TET_UNINITIATED;
        break;
    
    case T2C_TR_TIME_EXPIRED:
        return T2C_TR_TIME_EXPIRED; // return as is
        break;
    
    case T2C_TR_FIP:
        return T2C_TR_FIP; // return as is
        break;
    
    default:
        // The test has probably called exit() with a non-standard code.
        return TET_UNRESOLVED;
        break;
    }
    
    return -1;
}

int 
t2c_printf(const char* format, ...)
{
    assert(format != NULL);
    
    int res = -1;
    char* fmt = NULL;
    va_list ap;
    
    fmt = (char*)strdup(format);
    if (fmt == NULL)
    {
        return -1;
    }
        
    va_start(ap, format);
    res = tet_vprintf(fmt, ap);
    va_end(ap);
    
    free(fmt);
    return res;
}

int 
t2c_print_func(const char* arg)
{
    assert(arg != NULL);
    return t2c_printf("%s", arg);
}

void
t2c_print_error(const char* test_name, const char* msg)
{
    assert(test_name != NULL);
    assert(msg != NULL);
    t2c_printf("%s: %s", test_name, msg);
    return;
}    

/////////////////////////////////////////////////////////////////////////////
