/// about.h
/// Contains various information about the program: name, version, copyright, 
/// usage info, etc.
/// This data is located in a single header to avoid cluttering of the source 
/// files with it.

/////////////////////////////////////////////////////////////////////////////
// Copyright 2009-2010 
// Institute for System Programming of the Russian Academy of Sciences (ISPRAS)
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
//    http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License. 
/////////////////////////////////////////////////////////////////////////////

#ifndef __T2C_ABOUT_H_1359_INCLUDED__
#define __T2C_ABOUT_H_1359_INCLUDED__

// config.h will be generated by 'configure' script. 
// PACKAGE_* constants are defined in config.h
// If, for any reason, config.h is not available or is incomplete, we 
// provide reasonable default values for the parameters below.
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

// Short name of the tool
#ifdef PACKAGE_NAME
#define T2C_SHORT_NAME PACKAGE_NAME
#else 
#define T2C_SHORT_NAME \
    "t2c"
#endif    

// Codename (should be changed when the major or minor version of T2C changes)
// See "The Silmarillion" by J.R.R.T.
#define T2C_CODENAME \
    "Arnor"

// Version
#define T2C_VERSION \
    "2.0.0 \"" T2C_CODENAME "\" final"

// Long name of the tool
#define T2C_LONG_NAME \
    "T2C (\"Template-to-Code\") File Generator"

// Email address to report bugs to
#ifdef PACKAGE_BUGREPORT
#define T2C_BUGREPORT PACKAGE_BUGREPORT
#else 
#define T2C_BUGREPORT \
    "http://sourceforge.net/projects/template2code/develop"
#endif 

// Copyright notice
#define T2C_COPYRIGHT \
    "Copyright 2009-2010\n" \
    "Institute for System Programming of the Russian Academy of Sciences (ISPRAS)\n"    

// Short license notice (see LICENSE for details)
#define T2C_LICENSE \
    T2C_COPYRIGHT \
    " \n"	\
    " Licensed under the Apache License, Version 2.0 (the \"License\").\n"	\
    " You may obtain a copy of the License at\n"	\
    " \n"	\
    "    http://www.apache.org/licenses/LICENSE-2.0\n"	\
    " \n"	\
    " Unless required by applicable law or agreed to in writing, software\n"	\
    " distributed under the License is distributed on an \"AS IS\" BASIS,\n"	\
    " WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.\n"	\
    " See the License for the specific language governing permissions and\n"	\
    " limitations under the License. \n"	

// Usage information
#define T2C_USAGE                              \
    "Usage: t2c [options] <suite_root_dir> [<t2c_file_path>]\n"   \
    "Options:\n"                               \
    "  -d <dir>, --templates=<dir>\n" \
    "              Load templates from directory <dir> instead of the default one.\n" \
    "  -t <target>, --target=<target>\n" \
    "              Generate files for the specified target.\n" \
    "  -f <file>, --list-file=<file>\n" \
    "              Define TEST_MODULE attribute and populate it with the records from\n" \
    "              <file>. Can be useful to create the list of test executables, etc.\n" \
    "  --version   Display version information and then exit.\n" \
    "  --print-data-dir\n" \
    "              Display the path to the data directory of T2C and then exit.\n" \
    "  --help      Display this information and then exit.\n" \
    "\n" \
    "T2C uses special templates to generate files for a test suite: test sources, " \
    "makefiles, etc. Test sources for a target programming language are created " \
    "based on the tests in T2C format - .t2c files (see T2C Reference for details)." \
    "\n\n" \
    "Typically, one may need to execute this program several times:\n" \
    "- first run it without <t2c_file_path> argument to create common files for " \
    "the test suite (\"common\" mode);\n" \
    "- then run it for each .t2c file in the suite to create test-specific files " \
    "(\"test\" mode).\n" \
    "\n" \
    "<t2c_file_path> may specify a location outside of <suite_root_dir>. " \
    "\n\n" \
    "<suite_root_dir> specifies the root directory of the test suite. It contains " \
    "a configuration file and a number of other files and directories " \
    "(see T2C Reference for details)." \
    "\n\n" \
    "If -d (--templates) option is present, the directory specified there is " \
    "used as a main template directory instead of the default one." \
    "\n\n" \
    "If -t (--target) option is present, the files are generated for this target " \
    "instead of the one specified in TARGET parameter in the configuration " \
    "file of the suite." \
    "\n\n" \
    "Generally, the templates are loaded from the following directory:\n" \
    "<template_directory>/<target>/<mode>, where <mode> is \"common\" or \"test\" " \
    "(without quotes),\n" \
    "for example: /opt/t2c/share/templates/c_tet/common (target: c_tet, mode: common)" \
    "\n\n" \
    "Bug reporting:\n" \
    "if you have found a bug in T2C or want to make a feature request, " \
    "the tracker is available at\n" \
    T2C_BUGREPORT \
    ""

#endif // __T2C_ABOUT_H_1359_INCLUDED__
