// main.c
// Provides main() function for T2C test code generator 

/////////////////////////////////////////////////////////////////////////////
// Copyright 2009-2010 
// Institute for System Programming of the Russian Academy of Sciences (ISPRAS)
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
//    http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License. 
/////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "t2c.h"

///////////////////////////////////////////////////////////////////////////
// Globals
///////////////////////////////////////////////////////////////////////////
struct SSettings settings = {
    NULL,   // t2c_data_dir
    NULL,   // template_dir
    NULL,   // suite_root_dir
    NULL,   // t2c_file_path
    NULL,   // t2c_file_name
    NULL,   // target
    NULL,   // test_group_params
    NULL,   // aux_template_dir
    NULL,   // per_test_template_dir
    NULL,   // template_list
    NULL    // test_name_file_path
};

///////////////////////////////////////////////////////////////////////////  
// Main
///////////////////////////////////////////////////////////////////////////
int
main(int argc, char* argv[])
{
    if (init(argc, argv, &settings) == 0)
    {
        cleanup_settings(&settings);
        return EXIT_FAILURE; 
    }
    
    if (load_configuration(&settings) == 0)
    {
        cleanup_settings(&settings);
        return EXIT_FAILURE; 
    }
   
    // load templates taking overriding into account
    settings.template_list = load_template_list(&settings);
    if (settings.template_list == NULL)
    {
        cleanup_settings(&settings);
        return EXIT_FAILURE; 
    }
    
    // generate files (take mode into account)
    if (!generate_files(&settings))
    {
        cleanup_settings(&settings);
        return EXIT_FAILURE;
    }
   
    cleanup_settings(&settings);
    return EXIT_SUCCESS;
}
