///////////////////////////////////////////////////////////////////////////
// This file is generated for <$TARGET$> target. 
// 
// This target is designed to provide a rather minimal program for debugging
// of the source of the test or for analysing the behaviour of the system
// being tested. 
// The goal is to provide just a C/C++ source + makefile for it. That is, 
// no test harness, only a plain C or C++ program. 
//
// Therefore a lot of features is not supported here including requirement
// catalogues, execution in separate processes, wait time, etc. 
// 
// Current directory is used for test data (see the definition of
// T2C_GET_DATA_PATH() below).
//
// Again, there is little here except the contents of the corresponding
// t2c-file.
//
// This file was created using T2C ("Template-to-Code") system:
// http://sourceforge.net/projects/template2code/
///////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////
// To execute all the tests defined in this file, just run 
//     './<$T2C_FILE_NAME$>'
// without any arguments.
// 
// To execute only the tests with specific indices, specify these indices 
// as the arguments separated by spaces, for example: 
//     './<$T2C_FILE_NAME$> 10 3 9 1 5'
// Here the tests #10, #3, #9, #1 and #5 will be executed (in this order).
// 
// An index of a test is a 1-based number - it can be found in the comment 
// block before the test (see "This is the test #...").
///////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////
// If it is necessary to obtain a program that just executes the particular
// tests, one can replace the part of main() function between startup_func()
// and cleanup_func() function calls simply with calls of the test functions
// of interest.
// 
// One can also remove 'tf_list_' array from this file in this case.
///////////////////////////////////////////////////////////////////////////

#include <errno.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

///////////////////////////////////////////////////////////////////////////
// Common definitions - minimal implementation of T2C API
#define FALSE 0
#define TRUE  1
    
#define TRACE(fmt, ...) {printf(fmt, __VA_ARGS__); printf("\n");}
#define TRACE0(fmt) TRACE("%s", fmt)

#define BEGIN_FINALLY_SECTION test_finally_section_:
#define END_FINALLY_SECTION

#define RETURN if (in_finally_ == 0) { goto test_finally_section_; } else { return; } 
    
#define TODO_REQ() (TRUE)
#define REQ(r_id, r_comment, r_expr) { \
    if(!(r_expr)) { \
        TRACE("Failure at line %u: (%s) is FALSE", __LINE__, #r_expr); \
        RETURN; \
    } \
}
#define REQva(r_id, r_expr, r_fmt, ...) { \
    int expr_ = (r_expr); \
    if (!expr_) { \
        TRACE(r_fmt, __VA_ARGS__); \
    } \
    REQ(r_id, "", expr_);\
}

#define T2C_GET_DATA_PATH(rel_path) \
    (strlen(rel_path) == 0 ? (char*)strdup(".") : (char*)strdup(rel_path))
#define T2C_GET_TEST_PATH() \
    ((char*)strdup("./<$T2C_FILE_NAME$>"))

#define INIT_FAILED(msg) { \
    fprintf(stderr, "Initialization failed: %s\n", msg); \
    exit(EXIT_FAILURE); \
}

#define ABORT_TEST_PURPOSE(msg) { TRACE("Error: %s", msg); RETURN; }
#define ABORT_TEST(msg)         ABORT_TEST_PURPOSE(msg)
#define ABORT_UNSUPPORTED(msg)  { TRACE0(msg); RETURN; }
#define ABORT_UNTESTED(msg)     { TRACE0(msg); RETURN; }
#define ABORT_FIP(msg)          { TRACE0(msg); RETURN; }
#define TEST_FAILED(msg) { TRACE("Failure at line %u: %s", __LINE__, msg); RETURN; }

///////////////////////////////////////////////////////////////////////////
// Type of a test function.
typedef void (*FT2CTestFunc)();

///////////////////////////////////////////////////////////////////////////
// Globals
<$global$>

///////////////////////////////////////////////////////////////////////////
// Startup and cleanup

void 
startup_func()
{
    // user-defined startup instructions (if any)
    <$startup$>
    // end of user-defined startup instructions
    return;
}

void 
cleanup_func()
{
    // user-defined cleanup instructions (if any)
    <$cleanup$>
    // end of user-defined cleanup instructions
    return;
}

///////////////////////////////////////////////////////////////////////////
// The tests

<$test_case : join(\n)$>

///////////////////////////////////////////////////////////////////////////
// The list of the test functions
FT2CTestFunc tf_list_[] = {
<$tf_list_item : join(,\n)$>
};

///////////////////////////////////////////////////////////////////////////
int
main(int argc, char* argv[])
{
    startup_func();

    // total number of test functions in this file
    const unsigned long tf_num_ = <$num_tests$>;
    if (argc == 1) // no test indices specified, executing all the tests
    {
        for (int i = 0; i < tf_num_; ++i)
        {
            tf_list_[i]();
        }
    }
    else // executing the requested tests only
    {
        for (int p = 1; p < argc; ++p)
        {
            // try to convert the argument to number
            char* res = NULL;
            errno = 0;
            
            unsigned long ti = strtoul(argv[p], &res, 10); // test index
            if (errno != 0 || *res != '\0')
            {
                fprintf(stderr, "Warning: failed to convert the following string to a number: \"%s\"\n",
                    argv[p]);
            }
            else if (ti < 1 || ti > tf_num_)
            {
                fprintf(stderr, "Warning: there is no test with index %lu\n", ti);
            }
            else // a valid index has been specified
            {
                tf_list_[ti - 1]();
            }
        }
    }

    cleanup_func();
    return 0;
}
