/**************************************************************************
Copyright (C) <$YEAR$> <$COPYRIGHT_HOLDER$>. All rights reserved.

<$LICENSE$>
**************************************************************************/

///////////////////////////////////////////////////////////////////////////
// This file contains tests for the following library: 
//     <$library$>, 
// section: 
//     "<$libsection$>"
// 
// Test count: <$num_tests$>
//
// This file was created using T2C ("Template-to-Code") system:
// http://sourceforge.net/projects/template2code/
///////////////////////////////////////////////////////////////////////////

#include <errno.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <assert.h>

#include <t2c_api.h>

// 'suite_root_dir_' will be filled with an absolute path corresponding to
// 'suite_root_dir_rel_'. This absolute path will be used by T2C_GET_DATA_PATH
// to produce absolute paths to the specified files, etc.
//
// 'suite_root_dir_rel_' is relative to the directory containing this 
// executable.
const char* suite_root_dir_rel_ = "../..";
char* suite_root_dir_ = NULL;

const char* test_name_ = "<$T2C_FILE_NAME$>";
const char* test_path_rel_ = "tests/<$T2C_FILE_NAME$>/<$T2C_FILE_NAME$>";
const char* jrnl_path_rel_ = "tests/<$T2C_FILE_NAME$>/<$T2C_FILE_NAME$>.log";
const char* sep_line_ = 
    "---------------------------------------------------------------------------";

const int num_tests_ = <$num_tests$>;
int tests_passed_ = 0;

unsigned int wait_time_ = 30;
int single_process_ = 0;     

#ifdef CHECK_EXT_REQS
    int check_ext_ = 1;
#else
    int check_ext_ = 0;
#endif

int init_failed_ = FALSE;
char* init_fail_reason_ = NULL;

///////////////////////////////////////////////////////////////////////////
// The directory where the files containing requirements are located.
char* rcat_dir_ = NULL;
const char* rcat_dir_rel_ = "../../reqs";


// Catalogue(s) of requirements to be used (empty strings are ignored):
const char* rcat_names_[] = {
    "<$T2C_FILE_NAME$>",
    <$if concat(req_cat)$>"<$req_cat : join(",\n    ")$>",
    <$endif$>NULL
};

// Full catalogue of requirements to be used in the tests.
CReqCat* rcat_ = NULL;

///////////////////////////////////////////////////////////////////////////
// Utility functions - declarations
static void 
test_launcher(int test_index);

static void 
startup_func();

static void 
cleanup_func();

static void
special_cleanup_func();

///////////////////////////////////////////////////////////////////////////
// User-defined globals 
<$global$>

///////////////////////////////////////////////////////////////////////////
// Startup and cleanup functions

static void 
startup_func()
{
    const char* reason_to_cancel_ = NULL;
    init_failed_ = FALSE;
    t2c_printf_noindent("[Startup begins]");

    // initialize common settings: wait time, etc.
    wait_time_ = (unsigned int)t2c_get_number("<$WAIT_TIME$>", 
        (long)wait_time_, test_name_, t2c_print_func);
    single_process_ = (int)t2c_get_number("<$SINGLE_PROCESS$>", 
        (long)single_process_, test_name_, t2c_print_func);
    
    // load the requirements
    char* err_ = NULL;
    rcat_ = t2c_rcat_create(rcat_names_, rcat_dir_, &err_);
    if (rcat_ == NULL)
    {
        if (err_)
        {
            TRACE("%s: %s", test_name_, err_);
            free(err_);
            err_ = NULL;
        }
        fprintf(stderr, "\n%s: failed to load catalogue of requirements\n",
            test_name_);
        
        INIT_FAILED("<$T2C_FILE_NAME$>: failed to load catalogue of requirements");
    }

    // perform user-defined startup instructions (if any)
    if (!init_failed_)
    {
    <$startup$>
    }
    // end of user-defined startup instructions    
    
    if (init_failed_)
    {
        assert(reason_to_cancel_ != NULL);
        init_fail_reason_ = (char*)strdup(reason_to_cancel_);

        if (init_fail_reason_ == NULL)
        {
            // Out of memory. Do some cleaning-up and exit. 
            // Resource leaks are likely in this case: user-defined
            // cleanup instructions will not be executed.
            if (rcat_ != NULL)
            {
                t2c_rcat_destroy(rcat_);
            }
            fprintf(stderr, "\nOut of memory\n");
            exit(EXIT_FAILURE);
        }

        TRACE0("Initialization of the tests failed:");
        TRACE("%s", init_fail_reason_);
        
        fprintf(stderr, 
            "\n%s: initialization failed, see the journal for details\n", 
            test_name_);
    }
    else
    {
        fprintf(stderr, "\nExecuting tests for %s\n", test_name_);
    }
    
    t2c_printf_noindent("[Startup ends]");
    t2c_printf_noindent("%s", sep_line_);
    return;
}

static void 
cleanup_func()
{
    t2c_printf_noindent("[Cleanup begins]");
    if (!init_failed_)
    {
        fprintf(stderr, "\nPassed %d of %d\n", tests_passed_, num_tests_);
    }
    
    // perform user-defined cleanup instructions (if any)
    <$cleanup$>
    // end of user-defined cleanup instructions

    if (rcat_ != NULL)
    {
        t2c_rcat_destroy(rcat_);
    }
    free(init_fail_reason_);
    
    t2c_printf_noindent("[Cleanup ends]");
    t2c_printf_noindent("%s", sep_line_);
    return;
}

///////////////////////////////////////////////////////////////////////////
// The tests.
// The name of each test function has the following form:
//     test_<block_index>_<test_index>
// <block_index> is a 1-based index of the <BLOCK> section in the .t2c-file
//     the test was generated from;
// <test_index> is a 1-based index of the test in that <BLOCK> section.
///////////////////////////////////////////////////////////////////////////

<$test_case : join(\n)$>
    
///////////////////////////////////////////////////////////////////////////
// Specials
///////////////////////////////////////////////////////////////////////////

// The list of test functions in this file. 
FT2CTestFunc test_func[] = {
    <$test_func_item : join(,\n    )$>,    
    NULL
};

// Test launcher.
static void 
test_launcher(int test_index)
{
    ET2CTestResult tres = T2C_TR_ERROR;
    char* err = NULL;
    t2c_printf_noindent("[Test #%d begins]", test_index + 1);
    t2c_log_flush();
    
    int result = t2c_spawn(
        // a test to launch
        test_func[test_index], 
        // no special startup function
        NULL,   
        // special cleanup function to be called once per test
        special_cleanup_func,
        // wait time (seconds)
        wait_time_,
        // if nonzero, the same process is used for all the tests
        single_process_,
        // result code of the test
        &tres,
        // error message
        &err);                  
    
    // if the result is T2C_TR_ERROR, there is an error in the template or 
    // API library: this result should have been already handled
    assert(tres != T2C_TR_ERROR);
        
    if (result == 0 && err != NULL)
    {
        t2c_print_error(test_name_, err);
        free(err);
    }
    if (tres == T2C_TR_PASSED)
    {
        ++tests_passed_;
    }
    
    t2c_printf_noindent("[Test #%d ends]", test_index + 1);
    t2c_printf_noindent("[Result] %s", t2c_get_result_string(tres));
    t2c_printf_noindent("%s", sep_line_);
    
    // output a marker
    fprintf(stderr, ".");
    return;
}

static void
special_cleanup_func()
{
    t2c_log_flush();
    return;
}

///////////////////////////////////////////////////////////////////////////
int
main(int argc, char* argv[])
{
    // first of all, prepare absolute path to the suite root directory
    char* tdir = t2c_get_dir(argv[0]);
    if (tdir == NULL)
    {
        fprintf(stderr, 
            "Error: failed to get the path to the base directory\n");
        exit(EXIT_FAILURE);
    }
    
    suite_root_dir_ = t2c_path_sum(tdir, suite_root_dir_rel_);
    if (suite_root_dir_ == NULL)
    {
        fprintf(stderr, 
            "Error: failed to get the path to the suite root directory\n");
        exit(EXIT_FAILURE);
    }
    
    // prepare other absolute paths
    rcat_dir_ = t2c_path_sum(tdir, rcat_dir_rel_);
    if (rcat_dir_ == NULL)
    {
        fprintf(stderr, 
            "Error: failed to get the path to the requirement catalogue directory\n");
        exit(EXIT_FAILURE);
    }
        
    // initialize the logging facility
    if (t2c_log_init(suite_root_dir_, jrnl_path_rel_) != 0)
    {
        fprintf(stderr, "Error: failed to initialize the logging facility\n");
        exit(EXIT_FAILURE);
    }
    
    // now t2c_printf* functions can be used
    t2c_printf_noindent("%s", sep_line_);
    char* test_path = T2C_GET_TEST_PATH();
    if (test_path == NULL)
    {
        fprintf(stderr, "Out of memory\n");
        exit(EXIT_FAILURE);
    }
    
    t2c_printf_noindent("Executing the tests from the following group:");    
    t2c_printf_noindent("%s", test_path);
    free(test_path);
    t2c_printf_noindent("%s", sep_line_);
    
    startup_func();
    
    if (!init_failed_)    
    {
        // total number of test functions in this file
        const unsigned long tf_num_ = <$num_tests$>;
        if (argc == 1) // no test indices specified, executing all the tests
        {
            for (int i = 0; i < tf_num_; ++i)
            {
                test_launcher(i);
            }
        }
        else // executing the requested tests only
        {
            for (int p = 1; p < argc; ++p)
            {
                // try to convert the argument to number
                char* res = NULL;
                errno = 0;
                
                unsigned long ti = strtoul(argv[p], &res, 10); // test index
                if (errno != 0 || *res != '\0')
                {
                    fprintf(stderr, "Warning: failed to convert the following string to a number: \"%s\"\n",
                        argv[p]);
                }
                else if (ti < 1 || ti > tf_num_)
                {
                    fprintf(stderr, "Warning: there is no test with index %lu\n", ti);
                }
                else // a valid index has been specified
                {
                    test_launcher(ti - 1);
                }
            }
        }
    }
    cleanup_func();
    t2c_log_fini();
    
    free(rcat_dir_);
    free(suite_root_dir_);
    free(tdir);
    return 0;
}
